function fusionresult = lp_sr_fusion_singleband( M1,M2,lplevel,h_rule,bd,blocksize,epsilon )

%    M1 - input image A
%    M2 - input image B
%    lplevel - lp decomposition level
%    h_rule - fusion rule for high-pass
%          h_rule == 1: choose max(abs)
%          h_rule == 2: salience / match measure with threshold == .75 (as proposed by Burt et al)
%          h_rule == 3: choose max with consistency check (as proposed by Li et al)
%          h_rule == 4: simple choose max
%          h_rule == [0,1): linear combination of A and B
%    bd    -- blockdimension 
%    blocksize -- size of block
%    epsilon -- construction error threshold

[z1 s1] = size(M1);
[z2 s2] = size(M2);
if (z1 ~= z2) | (s1 ~= s2)
  error('Input images are not of same size');
end;

% moment matching
M2 = (M2 - mean(M2(:))) * std(M1(:))/std(M2(:)) + mean(M1(:));
% define filter 
w  = [1 4 6 4 1] / 16;

% cells for selected images
E = cell(1,lplevel);

% loop over decomposition depth -> analysis
for ii = 1:lplevel 
  % calculate and store actual image size 
  [z s]  = size(M1); 
  zl(ii) = z; sl(ii)  = s;
  
  % check if image expansion necessary 
  if (floor(z/2) ~= z/2), ew(1) = 1; else, ew(1) = 0; end;
  if (floor(s/2) ~= s/2), ew(2) = 1; else, ew(2) = 0; end;

  % perform expansion if necessary
  if (any(ew))
  	M1 = adb(M1,ew);
  	M2 = adb(M2,ew);
  end;	
  
  % perform filtering 
  G1 = conv2(conv2(es2(M1,2), w, 'valid'),w', 'valid');
  G2 = conv2(conv2(es2(M2,2), w, 'valid'),w', 'valid');
 
  % decimate, undecimate and interpolate 
  M1T = conv2(conv2(es2(undec2(dec2(G1)), 2), 2*w, 'valid'),2*w', 'valid');
  M2T = conv2(conv2(es2(undec2(dec2(G2)), 2), 2*w, 'valid'),2*w', 'valid');
 
  % select coefficients and store them

   E(ii) = {1*(M2-M2T)};
  
  % decimate 
  M1 = dec2(G1);
  M2 = dec2(G2);
end;

% select base coefficients of last decompostion stage (low-pass fusion)
M1 = sprase_fusion_singleband(M1,M2,bd,blocksize,epsilon);

% loop over decomposition depth -> synthesis
for iii = lplevel:-1:1
  % undecimate and interpolate 
  M1T = conv2(conv2(es2(undec2(M1), 2), 2*w, 'valid'), 2*w', 'valid');
  % add coefficients
  M1  = M1T + E{iii};
  % select valid image region 
  M1 	= M1(1:zl(iii),1:sl(iii));
end;

% copy image
fusionresult = M1;
end

